/* boot_inject-cert.c - inject a root certificate into grub and pass to kernel */
/*
 *  GRUB  --  GRand Unified Bootloader
 *  Copyright (C) 1999,2000,2001,2002,2003,2004,2005  Free Software Foundation, Inc.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
/*
 *  Modification History
 *  --------------------
 *  2013-05-23:  Reserve a space for root certificate and 
 *               pass the address to kernel.
 */

#ifdef GRUB_ENABLE_INJECT_CERT

#include "boot_inject-cert.h"

/* Return the addr we choose for certaddr with size indicated */
static void* find_space_in_memmap(unsigned int size)
{
    unsigned int first_addr = 0;
    unsigned int find_addr = 0;

    if(get_mmap_entry((void *)SCRATCHADDR, 0) != 0
            || *((int *)SCRATCHADDR) != 0)
        grub_printf (" Address Map BIOS Interface is present\n");

    grub_printf(" Lower memory: %uK, "
            "Upper memory (to first chipset hole): %uK\n",
            mbi.mem_lower, mbi.mem_upper);

    first_addr = mbi.mem_upper * 1024 >= size ? 
        (mbi.mem_upper * 1024  - size) : 0 ;  

    /* find the allowed highest suitable area */
    if((mbi.flags & MB_INFO_MEM_MAP) && (first_addr != 0)) {
        /* limit the bound to 2G */
        if (first_addr > 0x80000000 - size)
            first_addr = 0x80000000 - size;

        struct AddrRangeDesc *map = (struct AddrRangeDesc *)mbi.mmap_addr;
        unsigned int end_addr = mbi.mmap_addr + mbi.mmap_length;
        while(end_addr > (unsigned int)map) {
            /* found higher suitable area */
            if (map->Type == MB_ARD_MEMORY && size <= map->Length &&
                  find_addr < map->BaseAddr && map->BaseAddr <= first_addr) {
                /* use the high end */
                find_addr = map->BaseAddr + map->Length - size;
                if (find_addr >= first_addr) {
                    find_addr = first_addr;
                    break;
                }
            }
            map = ((struct AddrRangeDesc *) (((unsigned int) map) + 4 + map->size));
        }
    }

    /* we don't find any suitable addr, we choose a empirical addr */
    if(find_addr != 0)
        return (void *)find_addr;

    return (void *)DEFAULT_CERT_ADDR;
}

/* Append inject cert addr after kernel boot arg line.
 *
 * dest:   the address at the end of orig kernel boot arg.
 * maxlen: the left length of the kernel boot arg buffer.
 */
int kernel_arg_append_cert_addr(char *dest, int leftlen)
{
    int *memaddr;

    if (64 < leftlen) {
        memaddr = find_space_in_memmap(ROOT_CERT_SIZE);
        grub_memcpy(memaddr, root_cert_buffer, SIZE_CERT_PEM);

        grub_sprintf(dest, " ownercert_loader.root_cert=0x%x memmap=4k$0x%x ", 
                memaddr, memaddr);
        grub_printf("0x%x %s \n", (unsigned int)dest, (char*)dest);
        return grub_strlen(dest);
    } else {
        grub_printf("Can NOT insert the memory reserved \
                    parameter into cmdline \n");
        return 0;
    }
}

#endif /* #ifdef GRUB_ENABLE_INJECT_CERT */
